/* ###*B*###
 * ERIKA Enterprise - a tiny RTOS for small microcontrollers
 *
 * Copyright (C) 2002-2012  Evidence Srl
 *
 * This file is part of ERIKA Enterprise.
 *
 * ERIKA Enterprise is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation,
 * (with a special exception described below).
 *
 * Linking this code statically or dynamically with other modules is
 * making a combined work based on this code.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this code with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * ERIKA Enterprise is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 along with ERIKA Enterprise; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 * ###*E*### */

 /** @file   ee_tc27x_cstart.c
  *  @brief  The system startup code
  *
  *  The system startup code initializes the processor's registers
  *  and the application C variables.
  *  Has been created starting from cstart.c TASKING start-up code and has
  *  been custumed to compile with HIGHTEC GNUC compiler (v4.6.1.2) too.
  *  We used TASKING 4.0r1 as compiler and you can find the
  *  documentation pdf file for compiler at:
  *  http://www.tasking.com/support/tricore/tc_user_guide_v4.0.pdf
  *  In following comments, sometime, paragraph reference of this document
  *  are added. 
  *  For HIGHTEC documentation visit site: http://www.hightec-rt.com/
  *
  *  @author Errico Guidieri
  *  @date 2012
  */

#include <stdlib.h>

/* ERIKA CPU API + Internal inclusion */
#include "cpu/tricore/inc/ee_tc_internal.h"
/* Resolve ERIKA trap vectors. (They are inside EE_tc_trapvect.c) */
#include "cpu/tricore/inc/ee_tc_trap.h"
/* Resolve ERIKA interrupt vectors. (They are inside EE_tc_intvect.c) */
#include "cpu/tricore/inc/ee_tc_irq.h"
/* MCU defines to populate registers + PLL configuration utility */
#include "mcu/infineon_tc27x/inc/ee_tc27x_internal.h"
/* ENDINIT Support (Must be included only by one Kernel Module) */
#include "mcu/infineon_tc27x/inc/ee_tc27x_endinit.h"

#ifdef __TASKING__

#pragma weak   exit
#pragma extern _Exit
#pragma profiling off           /* prevent profiling information on cstart */
#pragma tradeoff 4              /* preset tradeoff level (for size) */
#pragma runtime B               /* disable runtime error checking for cstart */
#pragma nomisrac                /* disable MISRA-C checking */
#pragma noclear                 /* bss clearing not before cinit */

/* libc exit function remapping  */
#define EE_EXIT exit

/* far data access specificator */
#define EE_FAR __far

/* C initialization function for TASKING is inside libc */
#define EE_tc27x_cinit _c_init
/* C initialization function */
void EE_tc27x_cinit( void );

/* Utility macros to set registers: */

/* Two pre-processor steps to let macros explode */
#define EE_tc27x_extru2(src,start,size) \
  ({ int res; __asm volatile (" extr.u\t %0,%1,%2,%3" : "=d" (res) : "d" (src),\
  "i" (start),"i" (size) : "memory"); res; })
#define EE_tc27x_extru(src,start,size) EE_tc27x_extru2(src,start,size)

#define EE_tc27x_setareg2(areg,val) \
  __asm volatile ( "mov.aa " #areg ",%0"::"a"(&val) : "memory" )
#define EE_tc27x_setareg(areg,val) EE_tc27x_setareg2(areg,val)

#elif defined (__GNUC__)

extern void _exit (int status);

/* libc exit function remapping  */
#define EE_EXIT _exit

/* GNUC meaninless fucnctions or data specificators */
#define EE_FAR
#define JUMP

/* Utility macros to set registers: */

/* Two pre-processor steps to let macros explode */
#define EE_tc27x_extru2(src,start,size) \
  ({ int res; __asm volatile (" extr.u\t %0,%1,%2,%3" : "=d" (res) : "d" (src),\
  "i" (start),"i" (size) : "memory"); res; })
#define EE_tc27x_extru(src,start,size) EE_tc27x_extru2(src,start,size)

#define EE_tc27x_setareg2(areg,val) \
{ __asm volatile (" movh.a\t %%" EE_PREPROC_STRING(areg) ",\
  hi:" EE_PREPROC_STRING(val) "\n\tlea\t %%"EE_PREPROC_STRING(areg)",\
  [%%" EE_PREPROC_STRING(areg) "]lo:"EE_PREPROC_STRING(val)"" : : : "memory"); }
#define EE_tc27x_setareg(areg,val) EE_tc27x_setareg2(areg,val)

/* C initialization function */
#ifdef EE_EXECUTE_FROM_RAM
#define EE_tc27x_cinit() ((void)0)
#else /* EE_EXECUTE_FROM_RAM */
static void EE_tc27x_cinit( void );
#endif /* EE_EXECUTE_FROM_RAM */

__asm ("\t .weak _SMALL_DATA3_,_SMALL_DATA4_");

#else
#error Unsupported compiler!
#endif

extern EE_FAR EE_UINT32 EE_E_USTACK[];    /* user stack end */
/* extern EE_FAR EE_UINT32 EE_LC_UE_ISTACK[]; interrupt stack end */
extern EE_FAR void EE_INT_TAB ( void );   /* interrupt table */
extern EE_FAR void EE_TRAP_TAB( void );   /* trap table */
extern EE_FAR void * EE_SMALL_DATA;       /* centre of A0 addressable area */
extern EE_FAR void * EE_LITERAL_DATA;     /* centre of A1 addressable area */
#ifdef __TASKING__
extern EE_FAR void * EE_A8_DATA;          /* centre of A8 addressable area */
extern EE_FAR void * EE_A9_DATA;          /* centre of A9 addressable area */
#endif /* __TASKING__ */

#ifdef EE_C_INIT_TC
/* Specific core initialization function */
extern void EE_C_INIT_TC ( void );
#endif /* EE_C_INIT_TC */

/* passed to exit libc function */
extern int main( void );

/* start-up functions */
void __NEVER_INLINE__ JUMP EE_tc27x_start( void );
/* This MUST be inline because you cannot make a real function call
   before initialize CSAs lists */
__INLINE__ void __ALWAYS_INLINE__ EE_tc27x_csa_init( void );

/* Declare user start-up callback before endinit  */
#ifdef EE_START_UP_USER_ENDINIT
extern void EE_START_UP_USER_ENDINIT( void );
#endif /* EE_START_UP_ENDINIT */

#if defined(__TASKING__) && defined(EE_ERIKA_TRAP_HANDLING_OFF)
/* If ERIKA support for trap handling is turned off, but still user will
   use default boot, enable trap vector from the C-library. */

EE_COMPILER_EXTERN(_trapmmu)
EE_COMPILER_EXTERN(_trapprotection)
EE_COMPILER_EXTERN(_trapinstruction)
EE_COMPILER_EXTERN(_trapcontext)
EE_COMPILER_EXTERN(_trapbus)
EE_COMPILER_EXTERN(_trapassertion)
EE_COMPILER_EXTERN(_trapsystem)
EE_COMPILER_EXTERN(_trapnmi)
#endif /* __TASKING__ && EE_DEFAULT_TRAP_HANDLING_OFF */

#ifdef __TASKING__
/*******************************************************************************
 * _START() - Startup Code
 ******************************************************************************/
/* Labelled Pragmas are Bugged in TASKING 4.0r1 Compiler */
/* EE_PRAGMA_SECTION(_START,libc.reset) */
EE_DO_PRAGMA(section code libc.reset)
void _START( void )
{
  EE_tc27x_start();
}
EE_DO_PRAGMA(section code restore)
#elif defined (__GNUC__)

/*******************************************************************************
 * _START() - Startup Code
 ******************************************************************************/
#pragma section ".startup_code" ax 4
void RESET_(void);

void RESET_(void)
{
  __asm (".global _START");
  __asm (".word 0x00000000");
  __asm (".word 0xb3590070");
  __asm (".word 0x00000000");
  __asm (".word 0x00000000");
  __asm (".word 0x00000000");
  __asm (".word 0x00000000");
  __asm (".word 0x791eb864");
  __asm (".word 0x86e1479b");
  /* we must make a jump to cached segment, why trap_tab follow */
  __asm ("_START: ja  EE_tc27x_start");
}

/* we switch to normal region */
#pragma section

#endif /* __TASKING__ || __GNUC__ */

/*******************************************************************************
 * @brief startup code
 ******************************************************************************/
#define EE_TC_START_PSW         0x00000B80U
#define EE_TC_START_PSW_ISP     0x00000980U

void __NEVER_INLINE__ JUMP EE_TC27X_START( void )
{
  EE_UREG pcxi;

#if defined (__MULTI__) && defined(__IRQ_STACK_NEEDED__)
  EE_UINT32 isp;
#endif

  /* Load user stack pointer. First thing to do */
  EE_ADDR const sp = (EE_ADDR)
      ((EE_UINT32)(EE_E_USTACK) & EE_STACK_ALIGN);
  EE_tc_set_SP(sp);

  /* Do a dsync before changing any of the csfr values, thus any previous
   * background state gets flushed first. Required for applications that jump
   * to the reset address.
   */
  EE_tc_dsync();

  /* Set the PSW to its reset value in case of a warm start */
#if defined (__MULTI__) && defined(__IRQ_STACK_NEEDED__)
  /* clear PSW.IS */
  EE_tc_set_psw(EE_TC_START_PSW_ISP);
#else /* __MULTI__ &&  __IRQ_STACK_NEEDED__  */
  EE_tc_set_psw(EE_TC_START_PSW);
#endif /* __MULTI__ &&  __IRQ_STACK_NEEDED__  */

  /* Set the PCXS and PCXO to its reset value in case of a warm start */
  pcxi = EE_tc_get_pcxi();
  pcxi &= 0xFFF00000U;
  EE_tc_set_pcxi(pcxi);

  /*
   * Clear the ENDINIT bit in the WDT_CON0 register in order
   * to disable the write-protection for registers protected
   * via the EndInit feature (for example: WDT_CON1).
   */
  EE_tc27x_endinit_set(EE_TC_ENDINIT_DISABLE);

  /*
   * Disable the Watchdog if requested. Watchdog is enabled by default.
   * The Watchdog is disabled after ENDINIT is set by endinit_set().
   */
  EE_WDTCPUCON1.U |= 0x8;

  /*
   * Clear the ENDINIT bit in the WDTSCON0 register in order
   * to disable the write-protection for safety-critical registers
   * protected via the safety EndInit feature.
   */
  EE_tc27x_safety_endinit_set(EE_TC_ENDINIT_DISABLE);

  /*
   * Disable the safety watchdog if requested. Safety watchdog
   * is enabled by default. The safety watchdog is disabled after
   * ENDINIT is set by safety_endinit_set().
   */
  SCU_WDTSCON1.U |= 0x8;

  /*
   * Load Base Address of Trap Vector Table.
   * Disable this if not started from RESET vector. (E.g.
   * ROM monitors require to keep in control of vectors)
   */
  __mtcr(BTV, (EE_UINT32)&EE_TRAP_TAB);

  /*
   * Load Base Address of Interrupt Vector Table.
   * Disable this if not started from RESET vector. (E.g.
   * ROM monitors require to keep in control of vectors)
   */
  __mtcr(BIV, (EE_UINT32)&EE_INT_TAB);

#ifdef EE_ICACHE_ENABLED
  /*
   * PCON0 configuration.
   */
  __mtcr(PCON0, 0U);
#endif

#ifdef EE_DCACHE_ENABLED
  /*
   * DCON0 configuration.
   */
  __mtcr(DCON0, 0U);
#endif

#if (!defined(__OO_BCC1__)) && (!defined(__OO_BCC2__)) && \
    (!defined(__OO_ECC1__)) && (!defined(__OO_ECC2__)) && (!defined(__AS_SC4__))
/* Clock Initialization needed for non OSEK Kernels */
#if defined(EE_CPU_CLOCK)
/******** Configure CCU Clock Control. ********/
  EE_tc27x_configure_clock_ctrl();

/********* Configure Oscillator Control. *********/
  EE_tc27x_configure_osc_ctrl();

/***************** Configure PLL *****************/

  /* The internal version of EE_tc27x_configure_clock is an inline
     function (to which the MCU API simple delegate) I use this version
     here because I cannot CALL a fuction before configuring CSA
     (thing done later) */
  EE_tc27x_configure_clock_internal(EE_CPU_CLOCK);

#ifdef __TASKING__
  /*
   * Enable the GTM to get MCS memory access,
   * required for MCS initialization which
   * is performed by _c_init. (TASKING)
   */
  GTM_CLC.U = 0U;
#endif /* __TASKING__ */
#endif /* !__OO_BCC1__ && !__OO_BCC2__ && !__OO_ECC1__ && !__OO_ECC2__ &&
  !__AS_SC4__ */


#endif /* EE_CPU_CLOCK */

  /*
   * Inititialize global address registers a0/a1 to support
   * __a0/__a1 storage qualifiers of the C compiler.
   */
  EE_tc27x_setareg(a0, EE_SMALL_DATA);

  EE_tc27x_setareg(a1, EE_LITERAL_DATA);
#ifdef __TASKING__
  /* NOT YET SUPPORTED BY OUR LINKER SCRIPTS FOR GNUC
   * Inititialize global address registers a8/a9 to support
   * __a8/__a9 storage qualifiers of the C compiler. A8 and A9
   * are reserved for OS use, or for application use in cases
   * where the application ans OS are tightly coupled.
   */

  EE_tc27x_setareg(a8, EE_A8_DATA);

  EE_tc27x_setareg(a9, EE_A9_DATA);
#endif /* __TASKING__ */

  /* Setup the context save area linked list. This MUST be inline because
     you cannot make a real function call before initialize CSAs lists */
  EE_tc27x_csa_init();

  /*
   * Initialize and clear C variables.
   */
  EE_tc27x_cinit();      /* initialize data */

#ifdef EE_C_INIT_TC
  EE_C_INIT_TC();      /* initialize core specific data */
#endif /* EE_C_INIT_TC */

#if defined (__MULTI__) && defined(__IRQ_STACK_NEEDED__)
  /*
   * Load interupt stack pointer.
   * Disable this if not started from RESET vector. (E.g.
   * ROM monitors require to keep in control of vectors)
   */
  /* EE_UINT32 isp = (EE_UINT32)(_lc_ue_istack) & EE_STACK_ALIGN; */
  isp = (EE_UINT32)EE_tc_IRQ_tos.SYS_tos & EE_STACK_ALIGN;
  __mtcr(ISP, isp);
#endif /* __MULTI__ &&  __IRQ_STACK_NEEDED__ */

  /*
   * Set the ENDINIT bit in the WDT_CON0 register again
   * to enable the write-protection.
   */
  EE_tc27x_endinit_set(EE_TC_ENDINIT_ENABLE);

  /*
   * Set the ENDINIT bit in the WDTSCON0 register to enable the
   * safety-critical register write-protection.
   */
  EE_tc27x_safety_endinit_set(EE_TC_ENDINIT_ENABLE);

#ifdef EE_START_UP_USER_ENDINIT
  /*  Call the user callback to let him do initial configuration with
      endinit & safe_endinit protections disabled. Beware that protected
      registers are unlocked for the duration of the Time-out
      Period only! */
  EE_tc27x_endinit_set(EE_TC_ENDINIT_DISABLE);
  EE_tc27x_safety_endinit_set(EE_TC_ENDINIT_DISABLE);
  EE_START_UP_USER_ENDINIT();
  EE_tc27x_endinit_set(EE_TC_ENDINIT_ENABLE);
  EE_tc27x_safety_endinit_set(EE_TC_ENDINIT_ENABLE);
#endif /* EE_START_UP_USER_ENDINIT */

  /*
   * Call C main program.
   */
  EE_EXIT( main() );
}

/**************************************************************************
 *
 * FUNCTION:     EE_tc27x_csa_init
 *
 * DESCRIPTION: Setup the context save area linked list. This MUST be
 *              inline because you cannot make a real function call before
 *              initialize CSAs lists
 *
 *************************************************************************/
__INLINE__ void __ALWAYS_INLINE__ EE_tc27x_csa_init( void )
{
  /* Context Save Area base */
  extern EE_CSA EE_B_CSA[];
  /* Context Save Area end  */
  extern EE_CSA EE_E_CSA[];
  /* Nr of CSAs in area 1      */
  EE_INT32 const no_of_csas = EE_E_CSA - EE_B_CSA;
  /* Current CSA segment (I handle only one segment) */
  EE_UINT32 const pcxi_seg = (((EE_UINT32)EE_E_CSA >> 28U) & 0xFU) << 16U;
  /* for cycle index */
  EE_UINT32 i;
  /* Current CSA offset (I handle only one segment) */
  EE_UINT32 pcxi_o;
  /* Previous Context Pointer (CSA Link Word) */
  EE_UINT32 pcxi_val = 0U;
  /*  This variable hold the number of CSA that I have to leave free to
      handle FCD trap 3 are needed by this trap and 3 more are needed for
      potential nested asynchronus trap.
      To handle a trap 3 CSA are needed because class trap handler entry
      call TNI switch routine that call TNI sub-handler.
  */
  EE_UINT32 fcd_needed_csa = 6U;
  /* Current CSA pointer */
  EE_CSA * csa;

  /* IMPORTANT:
     I initialize the Free Context List in direct order.
     So I will end with a list that will grow toward smaller addresses.
   */
  for (i = 0U; i < (EE_UINT32)no_of_csas; ++i)
  {
    /* Get th current CSA pointer */
    csa = &EE_B_CSA[i];
    /* Store in current CSA previous pointer
       (null pointer in last CSA = first time!) */
    csa->next.reg = pcxi_val;
    /* Evaluate the CSA Segment Offset */
    pcxi_o = (((EE_UINT32)csa >> 6U) & 0xFFFFU);
    /* Compose pcxi_seg and pcxi_o in the pcxi_val to be stored in next CSA */
    pcxi_val = pcxi_seg | pcxi_o;

    /* Check if you have to populate LCX that point to the 'almost empty'
       position  */
    --fcd_needed_csa;
    if (fcd_needed_csa == 0U)
    {
      __mtcr(LCX, pcxi_val);
    }
  }
  /* Initialize the HEAD of Free Context List */
  EE_tc_set_fcx(pcxi_val);
}

#if defined(__GNUC__) && (!defined(EE_EXECUTE_FROM_RAM))
/**************************************************************************
 *
 * FUNCTION:     EE_tc27x_cinit
 *
 * DESCRIPTION: Initialize C Data
 *
 *************************************************************************/
static void EE_tc27x_cinit( void ) {

  /* Clear, Copy Tables pointers */
  typedef volatile union
  {
    EE_UINT8  ucPtr;
    EE_UINT16 usPtr;
    EE_UINT32 uiPtr;
    EE_UINT64 ullPtr;
  } EE_TABLE_PTR;

  /* clear table entry */
  extern EE_UINT32 __clear_table[];
  /* copy table entry */
  extern EE_UINT32 __copy_table[];

  EE_UINT32 uiLength, uiCnt;
  EE_UINT32 *pTable;
  EE_TABLE_PTR *pBlockDest;
  EE_TABLE_PTR *pBlockSrc;

  /* clear table */
  pTable = __clear_table;
  while (pTable)
  {
    pBlockDest = (EE_TABLE_PTR *) * pTable;
    pTable++;
    uiLength = *pTable;
    pTable++;
    /* we are finished when length == -1 */
    if (uiLength == 0xFFFFFFFFU) {
      break;
    }
    uiCnt = uiLength / 8U;
    while (uiCnt) {
      (*pBlockDest).ullPtr = (EE_UINT64)0U;
      pBlockDest++;
      uiCnt--;
    }
    if ((uiLength) & 0x4U) {
      (*pBlockDest).uiPtr = (EE_UINT32)0U;
      pBlockDest++;
    }
    if ((uiLength) & 0x2U) {
      (*pBlockDest).usPtr = (EE_UINT16)0U;
      pBlockDest++;
    }
    if ((uiLength) & 0x1U) {
      (*pBlockDest).ucPtr = 0U;
    }
  }

  /* copy table */
  pTable = __copy_table;
  while (pTable)
  {
    pBlockSrc = (EE_TABLE_PTR *) * pTable;
    pTable++;
    pBlockDest = (EE_TABLE_PTR *) * pTable;
    pTable++;
    uiLength = *pTable;
    pTable++;
    /* we are finished when length == -1 */
    if (uiLength == 0xFFFFFFFFU) {
      break;
    }
    uiCnt = uiLength / 8U;
    while (uiCnt) {
      (*pBlockDest).ullPtr = (*pBlockSrc).ullPtr;
      uiCnt--;
      pBlockDest++;
      pBlockSrc++;
    }
    if ((uiLength) & 0x4U) {
      (*pBlockDest).uiPtr = (*pBlockSrc).uiPtr;
      pBlockDest++;
      pBlockSrc++;
    }
    if ((uiLength) & 0x2U) {
      (*pBlockDest).usPtr = (*pBlockSrc).usPtr;
      pBlockDest++;
      pBlockSrc++;
    }
    if ((uiLength) & 0x1U) {
      (*pBlockDest).ucPtr = (*pBlockSrc).ucPtr;
    }
  }
}
#endif /* __GNUC__ && !EE_EXECUTE_FROM_RAM */

